;***************************************************************************************************************

;This software is provided in an AS IS condition,NO WARRANTIES in any form apply to this software.

; picmicrolab.com 9.15.2017
;***************************************************************************************************************
;
;Bargraph Voltmeter VFD Display based on PIC16F877 microcontroller
;
;--------------------------------------------------------------------------------------------------------------	

		LIST 	P=PIC16F877
		include	P16f877.inc
 __CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF

; 	
;
VoltageBarValue EQU	0x60
LCD_Address EQU	0x61

		org		0x00
reset	goto	start

		org	0x10
start	bcf		STATUS, RP0
		bcf		STATUS, RP1		;Bank 0
		clrf	PORTD
		clrf	PORTE
		clrf 	PORTA

		bcf 	INTCON,GIE ;disable interrupts

		bsf		STATUS, RP0		;Bank 1
		movlw	0x02
		movwf	ADCON1

		movlw 	0xFF
		movwf 	TRISA ;PortA input

		clrf	TRISE		;porte output 
		clrf	TRISD		;portd output

		bcf		STATUS, RP0		;Bank 0

		movlw 	0x89
		movwf 	ADCON0 ;Fosc/32; channel_1; ADC on

		call	init
;-------------------------------
movlw 0x80
movwf LCD_Address
;-------------------------------
wait:

call adc
call ShowLCD_BarGraph

goto	wait

ShowLCD_BarGraph:

	movlw 0x80
	movwf LCD_Address

loop:
	movlw 0x0f
	bsf	STATUS,C
	subwf VoltageBarValue,f
	btfsc STATUS,C
	goto skip
	call Clear_lcd;Clear previous location
	return
;incf	LCD_Address
;----------------------------------------------------------	
skip:		;movlw	0xC4   ;PLACE for the data on the LCD
	
		movf	LCD_Address,w
		movwf	0x20
		call 	lcdc
		call	mdel

		;movlw	0x2A			; CHAR (the data )
		movlw	0xff
		movwf	0x20
		call 	lcdd
		call	mdel

		incf	LCD_Address
goto loop
;--------------------------------------------------------
return

;--------------------------------------------------------
Clear_lcd:
		;movlw	0x2A			; CHAR (the data )
	
clear:		
		movlw 0x8f
		subwf LCD_Address,w
		btfsc STATUS,Z
		return
		
		movlw	0xfe
		movwf	0x20
		call 	lcdd
		call	mdel
		
		incf	LCD_Address

		goto clear

return
;-------------------------------------------------------
adc:

		call d_20 ;Delay TACQ
loop_a: bsf ADCON0,GO ;start conversion
waitc: 	btfsc ADCON0,GO ;wait end of conversion
		goto 	waitc
		call 	d_4
		movf 	ADRESH,W
		movwf 	VoltageBarValue
		;goto 	loop_a
d_20: 	movlw 0x20
		movwf 0x22
loop_b: decfsz 0x22,1
		goto loop_b
		return
d_4: 	movlw 0x06
		movwf 0x22
loop_c: decfsz 0x22,1
		goto loop_c
		return

return
;--------------------------------------------------------
;
;subroutine to initialize LCD
;
init	movlw	0x30
		movwf	0x20
		call 	lcdc
		call	del_41

		movlw	0x30
		movwf	0x20
		call 	lcdc
		call	del_01

		movlw	0x30
		movwf	0x20
		call 	lcdc
		call	mdel

		movlw	0x01		; display clear
		movwf	0x20
		call 	lcdc
		call	mdel

		movlw	0x06		; ID=1,S=0 increment,no  shift 000001 ID S
		movwf	0x20
		call 	lcdc
		call	mdel

		movlw	0x0c		; D=1,C=B=0 set display ,no cursor, no blinking
		movwf	0x20
		call 	lcdc
		call	mdel

		movlw	0x38		; dl=1 ( 8 bits interface,n=12 lines,f=05x8 dots)
		movwf	0x20
		call 	lcdc
		call	mdel
		return
;
;subroutine to write command to LCD
;
lcdc	movlw	0x00		; E=0,RS=0 
		movwf	PORTE
		movf	0x20,w
		movwf	PORTD
		movlw	0x01		; E=1,RS=0
		movwf	PORTE
        call	sdel
		movlw	0x00		; E=0,RS=0
		movwf	PORTE
		return
;
;subroutine to write data to LCD
;
lcdd	movlw		0x02		; E=0, RS=1
		movwf		PORTE
		movf		0x20,w
		movwf		PORTD
        movlw		0x03		; E=1, rs=1  
		movwf		PORTE
		call		sdel
		movlw		0x02		; E=0, rs=1  
		movwf		PORTE
		return
;----------------------------------------------------------
del_41	movlw		0xcd
		movwf		0x23
loop6	movlw		0x20
		movwf		0x22
loop7	decfsz		0x22,1
		goto		loop7
		decfsz		0x23,1
		goto 		loop6 
		return

del_01	movlw		0x20
		movwf		0x22
loop8	decfsz		0x22,1
		goto		loop8
		return

sdel	movlw		0x19		; movlw = 1 cycle
		movwf		0x23		; movwf	= 1 cycle
loop2	movlw		0xfa
		movwf		0x22
loop1	decfsz		0x22,1		; decfsz= 12 cycle
		goto		loop1		; goto	= 2 cycles
		decfsz		0x23,1
		goto 		loop2 
		return

mdel	movlw		0x0a
		movwf		0x24
loop5	movlw		0x19
		movwf		0x23
loop4	movlw		0xfa
		movwf		0x22
loop3	decfsz		0x22,1
		goto		loop3
		decfsz		0x23,1
		goto 		loop4 
		decfsz		0x24,1
		goto		loop5
		return

		end
